@@ -40,11 +40,13 @@ public:
     ~Private();
 
     void initialize();
+    void clean();
     Adapter *findUsableAdapter();
 
     void _k_adapterAdded(const QDBusObjectPath &objectPath);
     void _k_adapterRemoved(const QDBusObjectPath &objectPath);
     void _k_defaultAdapterChanged(const QDBusObjectPath &objectPath);
+    void _k_adapterPoweredChanged(bool powered);
     void _k_propertyChanged(const QString &property, const QDBusVariant &value);
 
     void _k_bluezServiceRegistered();
@@ -69,9 +71,9 @@ Manager::Private::Private(Manager *q)
     if (QDBusConnection::systemBus().isConnected()) {
         QDBusReply<bool> reply = QDBusConnection::systemBus().interface()->isServiceRegistered("org.bluez");
 
-	if (reply.isValid()) {
-	    m_bluezServiceRunning = reply.value();
-	}
+        if (reply.isValid()) {
+            m_bluezServiceRunning = reply.value();
+        }
     }
 }
 
@@ -100,23 +102,46 @@ void Manager::Private::initialize()
             defaultAdapterPath = reply.value().path();
             if (!defaultAdapterPath.isEmpty()) {
                 m_defaultAdapter = new Adapter(defaultAdapterPath, m_q);
+                connect(m_defaultAdapter, SIGNAL(poweredChanged(bool)), m_q, SLOT(_k_adapterPoweredChanged(bool)));
                 m_adaptersHash.insert(defaultAdapterPath, m_defaultAdapter);
+                emit m_q->defaultAdapterChanged(m_defaultAdapter);
             }
         }
         const QVariantMap properties = m_bluezManagerInterface->GetProperties().value();
         const QList<QDBusObjectPath> adapters = qdbus_cast<QList<QDBusObjectPath> >(properties["Adapters"].value<QDBusArgument>());
-        if (adapters.count() == 1) {
-            return;
-        }
-        Q_FOREACH (const QDBusObjectPath &path, adapters) {
-            if (path.path() != defaultAdapterPath) {
-                Adapter *const adapter = new Adapter(path.path(), m_q);
-                m_adaptersHash.insert(path.path(), adapter);
+        if (adapters.count() > 1) {
+            Q_FOREACH (const QDBusObjectPath &path, adapters) {
+                if (path.path() != defaultAdapterPath) {
+                    Adapter *const adapter = new Adapter(path.path(), m_q);
+                    connect(adapter, SIGNAL(poweredChanged(bool)), m_q, SLOT(_k_adapterPoweredChanged(bool)));
+                    m_adaptersHash.insert(path.path(), adapter);
+                }
             }
         }
+
+        m_usableAdapter = findUsableAdapter();
+        emit m_q->usableAdapterChanged(m_usableAdapter);
     }
 }
 
+void Manager::Private::clean()
+{
+    qDebug() << "Private::clean";
+    delete m_bluezManagerInterface;
+    QHashIterator<QString, Adapter*> i(m_adaptersHash);
+    while (i.hasNext()) {
+        i.next();
+        Adapter *adapter = m_adaptersHash.take(i.key());
+        emit m_q->adapterRemoved(adapter);
+        delete adapter;
+    }
+
+    m_usableAdapter = 0;
+    m_defaultAdapter = 0;
+
+    emit m_q->usableAdapterChanged(0);
+    emit m_q->defaultAdapterChanged(0);
+}
 Adapter *Manager::Private::findUsableAdapter()
 {
     Adapter *const defAdapter = m_q->defaultAdapter();
@@ -137,9 +162,12 @@ void Manager::Private::_k_adapterAdded(const QDBusObjectPath &objectPath)
 {
     qDebug() << "Added: " << objectPath.path();
     Adapter *const adapter = new Adapter(objectPath.path(), m_q);
+    connect(adapter, SIGNAL(poweredChanged(bool)), m_q, SLOT(_k_adapterPoweredChanged(bool)));
+
     m_adaptersHash.insert(objectPath.path(), adapter);
     if (!m_defaultAdapter) {
         m_defaultAdapter = adapter;
+        emit m_q->defaultAdapterChanged(m_defaultAdapter);
     }
     if (!m_usableAdapter || !m_usableAdapter->isPowered()) {
         Adapter *const oldUsableAdapter = m_usableAdapter;
@@ -183,12 +211,35 @@ void Manager::Private::_k_defaultAdapterChanged(const QDBusObjectPath &objectPat
     Adapter *adapter = m_adaptersHash[objectPath.path()];
     if (!adapter) {
         adapter = new Adapter(objectPath.path(), m_q);
+        connect(adapter, SIGNAL(poweredChanged(bool)), m_q, SLOT(_k_adapterPoweredChanged(bool)));
         m_adaptersHash.insert(objectPath.path(), adapter);
     }
     m_defaultAdapter = adapter;
     emit m_q->defaultAdapterChanged(adapter);
 }
 
+void Manager::Private::_k_adapterPoweredChanged(bool powered)
+{
+    Adapter *adapter = qobject_cast< Adapter* >(m_q->sender());
+    if (!adapter) {
+        qWarning("adapterPoweredChanged was called from a null adapter");
+        return;
+    }
+
+    if (m_usableAdapter && adapter != m_usableAdapter) {
+        return;//The modified adapter is not the usable one, ignore the event (we do not care)
+    }
+
+    if (m_defaultAdapter && !powered) {
+        m_defaultAdapter = 0;
+        emit m_q->usableAdapterChanged(0);
+        return;
+    }
+
+    m_defaultAdapter = adapter;
+    emit m_q->usableAdapterChanged(m_defaultAdapter);
+}
+
 void Manager::Private::_k_propertyChanged(const QString &property, const QDBusVariant &value)
 {
     Q_UNUSED(property)
@@ -198,28 +249,13 @@ void Manager::Private::_k_propertyChanged(const QString &property, const QDBusVa
 void Manager::Private::_k_bluezServiceRegistered()
 {
     m_bluezServiceRunning = true;
-    if (!m_bluezManagerInterface) {
-        initialize();
-    }
+    initialize();
 }
 
 void Manager::Private::_k_bluezServiceUnregistered()
 {
-    QHashIterator<QString, Adapter*> i(m_adaptersHash);
-    while (i.hasNext()) {
-        i.next();
-        Adapter *adapter = m_adaptersHash.take(i.key());
-        emit m_q->adapterRemoved(adapter);
-        delete adapter;
-    }
-
-    m_usableAdapter = 0;
-    m_defaultAdapter = 0;
-
-    emit m_q->usableAdapterChanged(0);
-    emit m_q->defaultAdapterChanged(0);
-
     m_bluezServiceRunning = false;
+    clean();
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -267,6 +303,7 @@ Adapter *Manager::defaultAdapter()
         const QString adapterPath = d->m_bluezManagerInterface->DefaultAdapter().value().path();
         if (!adapterPath.isEmpty()) {
             d->m_defaultAdapter = new Adapter(adapterPath, const_cast<Manager*>(this));
+            connect(d->m_defaultAdapter, SIGNAL(poweredChanged(bool)), SLOT(_k_adapterPoweredChanged(bool)));
             d->m_adaptersHash.insert(adapterPath, d->m_defaultAdapter);
         }
     }
